# Circumstance monitor scenario for La Silla MASS-LITE as a piggy-back of
# the ESO-DIMM.
# Since weather is not of our interest (we can observe whatever weather is given 
# DIMM observes as well), we trace the sun height and, when it is low enough,
# initialize the getdimm.tcl GETDIMM component. Just as the first
# data point is obtained, we consider observations to be started. If the DIMM
# is no longer observing, we stop observations.
#

#getdimm_monit timer identifier:
set getdimm_after 0
#Time-keeper of last-detected bad weather conditions in seconds-1969:
set getdimm_badtime 0

# Stop-scenario procedure: called only from SV by stop_monitor procedure.
proc end {} {
	# some formal commands
	cmd CIRC stop now
	cmd CIRC park
}

# Poller of the GETDIMM component: depends on hsun_monit
proc getdimm_monit delay {
	global getdimm_after getdimm_badtime
	cmd GETDIMM get dimmrun object airmass seeing utc
	switch [GETDIMM dimmrun] {
		1 {
			# can we start observations?: hsun, cond and time-bad_time
			if {[CIRC hsun]<[CIRC hsun_lim] && \
				[expr [clock seconds] - $getdimm_badtime]>[GETDIMM waittime] \
				&& ! [is_observations_now]} {after 1 start_obs}
			}
		0 { 
			if {[is_observations_now]} {
				stop_obs
				set getdimm_badtime [clock seconds]
			}
		}
		default	{ error "Unrecognized DIMMRUN=[GETDIMM dimmrun]" }
	}

	if {[CIRC hsun]<[CIRC hsun_lim]} {
		if { [is_observations_now] } {
			set delay [GETDIMM pp_cond_obs]
		} else {
			set delay [GETDIMM pp_cond_nobs]
		}
		# schedule next monitor body call:
		if {$getdimm_after!=0} {
			set getdimm_after [after [expr round($delay*1000)] getdimm_monit $delay]
		}
	}
	# else - do not restart until next external start by hsun_monit
}

#hsun_monit timer identifier:
set hsun_after ""
# Flag of morning-after-observations:
set is_morning 0
# Poller of the sun altitude: starts/stops getdimm_monit
proc hsun_monit delay {
	global getdimm_after hsun_after is_morning
	cmd CIRC get hsun st ut
	if {[CIRC hsun]<[CIRC hsun_lim]} {
		# start dimm-getting monitor
		if {$getdimm_after==0} {
			cmd GETDIMM init
			set getdimm_after [after 1 getdimm_monit 1]
		}
	} else {
		# stop observations and dimm-getting monitor
		if {[is_observations_now]} {
			stop_obs
			set is_morning 1
			cmd GETDIMM park
			after cancel $getdimm_after
			set getdimm_after 0
		}
	}
	if { [CIRC hsun]>0 } {
		set delay [CIRC pp_hsun_day]
		if {$is_morning} {cmd MASS quit}
	} else {
		set delay [CIRC pp_hsun_night]
	}
	# schedule next monitor body call:
	set hsun_after [after [expr round($delay*1000)] hsun_monit $delay]
}

# For simulation needs: remove when doing real job!!!!
#cmd CIRC set mode=test
cmd CIRC set longitude="[CIRC longitude]" latitude="[CIRC latitude]"
cmd CIRC get status
initialize "CIRC GETDIMM"
#cmd CIRC init
# start circ monitor:
hsun_monit 1
