## Guiding test scenario. To be started from SV v0.12 or later.
## 
## Written  by A.Tokovinin, Nov. 11, 2003
## Cleaned  by N.Shatsky, Nov 17, 2003
########################################################################

# Global control variables:
set CENTERED 0
set TEND 0

########################################################################
# Import the configuration file parameters on place, to release the 
# CFG errors immediately
set work_zone [CENT work_zone]
set cent_zone [CENT cent_zone]
set alpha [CENT alpha]
set cent_period [CENT period]

########################################################################
# Procedures:

# Getting the new object and pointing to it
# Affects: OBJM(RA), OBJM(DEC), and TEND
# we do not care on which star the telescope is actually pointed, for these tests
proc GETOBJECT {} {
	global TEND
	add_log "GETOBJECT"
	cmd OBJM run ra dec tvis hourh object
	# If there is no object - nothing to observe!:
	if {[OBJM status]=="NOSTAR"} { set TEND 0; return }
	cmd TLSP SET ra="[OBJM ra]" dec="[OBJM dec]"
	cmd TLSP RUN
	set tvis [OBJM tvis]
	add_log "Tvis= $tvis seconds"
 	set TEND [expr [clock seconds]+[OBJM tvis]]
	cmd DIMM set object=[OBJM object]
}

##############################################################################

# One-step centering of the star
# Affects: CENTERED
proc GUIDER {} {
	global cent_zone work_zone alpha CENTERED
	global guider_after

	# Get the shifts:
	cmd CENT get dra ddec

	# if star is detected and shifts are not small - correct:
	if {[CENT dra]!=999 && [CENT ddec]!=999 && \
		([expr abs([CENT dra])]>$cent_zone || \
		[expr abs([CENT ddec])]>$cent_zone)} {
	
		# corrections: negative shifts slightly decreased:
        set corra [expr (-1)*[CENT dra] * $alpha]
		set cordec [expr (-1)*[CENT ddec] * $alpha]
	    
		# introduce corrections:
		cmd CORR run dra=$corra ddec=$cordec

		# Get the resulting shifts:
		cmd CENT get dra ddec
	}
	# success-flag: in current scenario it is only for information:
	set CENTERED [expr abs([CENT dra])<=$work_zone && abs([CENT ddec])<=$work_zone]
	add_log "CENTERED->$CENTERED"
}

##############################################################################
# Compute the time rest for observations, in [sec]
# @return time rest for observations in seconds
proc getwtime {} {
	global TEND
	if {[set t [expr $TEND - [clock seconds]]]>0} {
		return $t
	} else { return 0}
}

# Stop running (or not) detectors
proc STOP {} {
	global guider_after
	add_log "STOP"
	cmd DIMM stop now
}

#proc error_handler {code comp} {
#	add_log "Handling error $code caused by $comp: success"
#	return 0
#}
proc end {} {
	stop_park "DIMM TLSP"
}
########################################################################
## Now the scenario body itself goes:
########################################################################

cmd OBJM init longitude="[OBJM longitude]" latitude="[OBJM latitude]"
# Infinite loop of targets observations:
while 1 {
	GETOBJECT

	while {[set WTIME [getwtime]]>0} {
		add_log "$WTIME seconds left"
		GUIDER
		wait_sec $cent_period
	}
	
	STOP	
}

