## ESO MASS-LITE observations scenario. To be started from SV v0.12 or later.
## 
## Scenario uses GETDIMM component to know what ESO DIMM is doing now.
########################################################################


proc end {} {
	# End of observations procedure:
	stop_park MASS
}

set runcent 0

set is_massrunerror 0
# reserved-name procedure of errors handling:
proc error_handler {code cmp} {
add_log "EHANDLE: code=$code, cmp=$cmp"
	global runcent is_massrunerror
	# skip the ERFAT error from TURBINA during flux measurements:
	if {$cmp=="MASS" && $code=="ECMPFAT"} {
		if {$runcent} {
			add_log "Star is not within field of view!"
		} else {
			add_log "Run Measurement or SET OBJECT error!"
			set is_massrunerror 1
		}
		return 1
	} else {
		return 0
	}
}

########################################################################
## Now the scenario body itself goes:
########################################################################

# Infinite loop of targets observations:
set object -1

# Mass run-command identifier:
set mci -1

# Here we had to put scanning....which command?
set nmeas 0
while 1 {
	cmd GETDIMM get object dimmrun
	# we do not ask GETDIMM since getdimm_monit procedure in circumstance
	# monitor circmon.lasilla.tcl does it frequently. So we use results:
	if {[GETDIMM object]!=$object} {\
		# ESO DIMM has changed (or set) object: stop current accumulation
		if {$mci>=0} {cmd MASS stop now; set mci 0}
		# re-initialize MASS for the case of misfunctions on previous star
		if {[MASS status]!="PARKED"} {cmd MASS park}
		if {[GETDIMM object]==0} {
			# object is not in the MASS catalogue: wait for approp.
			wait_sec 60
			continue
		} else {
			cmd MASS init
			# wait until turbina finishes its init-job
			wait_sec 20
			cmd MASS set object=[GETDIMM object]
			# Run centering star and flux estimation:
			set object [GETDIMM object]
			set nmeas 0
		}
	} else {
		if {$mci>=0 && ![is_cmd $mci]} {
			# already something registered:
			if {$is_massrunerror} {
				# wait: maybe error cause will disappear
				wait_sec 10
			} else {
				cmd MASS get integral
				cmd GETDIMM get putmass
				if {![GETDIMM putmass]} {
					add_log "Cannot put mass-file to FTP"
				}
			}
		}
	}
	if {[GETDIMM dimmrun]=="1" && ![is_cmd $mci]} {
		if {![expr $nmeas % 10]} {
			set runcent 1
			cmd MASS run scen2
			set runcent 0 
		}
		set is_massrunerror 0
		set mci [cmd MASS run &]
		incr nmeas
	} else {
		# wait a bit...
		wait_sec 2 0
	}
}

